wm withdraw .
set pi 3.1415

# Creating a circle with 12 segments
# First the points
set pts [ vtkPoints  New ]
$pts SetNumberOfPoints 9
for { set i 0 } { $i <= 7 } { incr i } {
   set rad [ expr 2.0*$i*$pi/8.0 ]
   set x [ expr 10.0* sin($rad) ]
   set y [ expr 10.0* cos($rad) ]
   $pts SetPoint $i $x $y 0.0
   puts stdout "Point $i, angle=$rad, point = [ $pts GetPoint $i ]"
}

# Set apex point as point 8
$pts SetPoint 8 0.0 0.0 10.0

# Next the line segments
set triangles [ vtkCellArray  New ]
$triangles Allocate 10 5
for { set i 0 } { $i <= 7 } { incr i } {
   set p1 $i
   set p2 [ expr $i + 1 ]
   if { $p2 > 7 } { set p2 0 }
   $triangles InsertNextCell 3
   $triangles InsertCellPoint $p1;
   $triangles InsertCellPoint $p2
   $triangles InsertCellPoint 8
   puts stdout "Set Line Segment $i = ($p1, $p2)"
}

# Create the curve object and set the points and triangles
set surface [ vtkPolyData  New ]
$surface SetPoints $pts
$surface SetPolys $triangles

# Since VTK uses reference counting delete pts and triangles
# as these are counted inside surface
$pts Delete
$triangles Delete

# Create the mapper
set map [ vtkPolyDataMapper   New ]
$map SetInput  $surface

#Create the actor and set it to display wireframes
set actor [ vtkActor   New ]
$actor SetMapper $map
$map Delete
set property [ $actor GetProperty ]
$property SetColor  1 1 1 ; $property SetAmbient 1.0
$property SetDiffuse 0.0;   $property SetSpecular 0.0
$property SetRepresentationToWireframe

# Create the renderer
set ren [ vtkRenderer  New ]
$ren AddActor $actor

# Set camera mode to Orthographic as opposed to Perspective
[ $ren GetActiveCamera ] ParallelProjectionOn

# Rest is standard window/interactor etc.
# Render Window 
set renWin [ vtkRenderWindow  New ]
$renWin AddRenderer $ren
$renWin SetSize 300 300

# Interactor
set iren [ vtkRenderWindowInteractor  New ]
$iren SetRenderWindow $renWin
$iren Initialize
$iren AddObserver SetExitMethod { exit }

$ren ResetCamera

# prevent the tk window from showing up then start the event loop
vwait forever


