

wm geometry . 200x200

# This is a simple volume rendering example that
# uses a vtkVolumeRayCast mapper

# Create the standard renderer, render window
# and interactor

# Create the reader for the data
# First Load the Image
set reader [ vtkStructuredPointsReader  New ] 
$reader SetFileName brain.vt
$reader Update

# Convert image to unsigned char (volume rendering requires unsigned char/short)
set cast [ vtkImageCast  New ]
$cast SetInput [ $reader GetOutput ]
$cast SetOutputScalarTypeToUnsignedChar

# Create transfer mapping scalar value to opacity
set opacityTransferFunction [ vtkPiecewiseFunction  New ]
$opacityTransferFunction AddPoint  40   0.0
$opacityTransferFunction AddPoint  255  1.0

# Create transfer mapping scalar value to color
set  colorTransferFunction [ vtkColorTransferFunction  New ]
$colorTransferFunction AddRGBPoint      0.0 0.0 0.0 0.0
$colorTransferFunction AddRGBPoint    255.0 1.0 1.0 1.0

# The property describes how the data will look
set volumeProperty [ vtkVolumeProperty  New ]
$volumeProperty SetColor $colorTransferFunction
$volumeProperty SetScalarOpacity $opacityTransferFunction
$volumeProperty SetInterpolationTypeToLinear

# The mapper / ray cast function know how to render the data
set volumeMapper [ vtkFixedPointVolumeRayCastMapper  New ]
$volumeMapper SetInput [ $cast GetOutput]

# The volume holds the mapper and the property and
# can be used to position/orient the volume
set volume [ vtkVolume  New ]
$volume SetMapper   $volumeMapper
$volume SetProperty $volumeProperty

set ren [ vtkRenderer  New ] 
$ren AddVolume $volume

set renwin [ vtkRenderWindow  New ]
$renwin AddRenderer $ren
$renwin SetSize 300 300
$ren ResetCamera
$renwin Render
$renwin SetDesiredUpdateRate 1.0

set iren [ vtkRenderWindowInteractor  New ]
$iren SetRenderWindow $renwin
$iren Initialize
$iren AddObserver SetExitMethod { exit }

wm withdraw .; vwait forever




