// This corresponds to script10-3.tcl

#include <vtkPoints.h>
#include <vtkCellArray.h>
#include <vtkPolyData.h>
#include <vtkPolyDataWriter.h>
#include <vtkMath.h>

int main(int argc,char *argv[])
{
  double pi=vtkMath::Pi();

  vtkPoints* pts=vtkPoints::New();
  pts->SetNumberOfPoints(8);
  for (int i=0;i<=7;i++)
    {
      double rad= 2.0*double(i)*pi/8.0;
      double x  = 10.0* sin(rad);
      double y  = 10.0* cos(rad);
      pts->SetPoint(i,x,y,0.0);
      double* p=pts->GetPoint(i);
      fprintf(stdout,"Point %d = (%.2f,%.2f,%.2f)\n",i,p[0],p[1],p[2]);
    }

  vtkCellArray* lines=vtkCellArray::New();
  lines->Allocate(10, 5);
  int p[2];
  for (int i=0;i<=7;i++)
    {
      p[0]=i;
      p[1]=i+1;
      if ( p[1] > 7)
	p[1]=0;
	  
      lines->InsertNextCell(2,p);
      fprintf(stdout,"Set Line Segment %d = (%d,%d)\n",i,p[0],p[1]);
    }

  vtkPolyData* curve=vtkPolyData::New();
  curve->SetPoints(pts);
  curve->SetLines(lines);


  // Since VTK uses reference counting delete pts and lines
  // as these are counted inside curve
  pts->Delete();
  lines->Delete();

  // Save the curve to a file
  vtkPolyDataWriter* writer=vtkPolyDataWriter::New();
  writer->SetInput(curve);
  writer->SetFileName("curve.vtk");
  writer->Write();

  // Clean Up all objects and exit
  writer->Delete();
  curve->Delete();

  return 0;
}
