// This corresponds to script10-4.tcl

#include <vtkPoints.h>
#include <vtkCellArray.h>
#include <vtkPolyData.h>
#include <vtkPolyDataWriter.h>
#include <vtkMath.h>

int main(int argc,char *argv[])
{
  double pi=vtkMath::Pi();

  vtkPoints* pts=vtkPoints::New();
  pts->SetNumberOfPoints(9);
  for (int i=0;i<=7;i++)
    {
      double rad= 2.0*double(i)*pi/8.0;
      double x  = 10.0* sin(rad);
      double y  = 10.0* cos(rad);
      pts->SetPoint(i,x,y,0.0);
      double* p=pts->GetPoint(i);
      fprintf(stdout,"Point %d = (%.2f,%.2f,%.2f)\n",i,p[0],p[1],p[2]);
    }
  pts->SetPoint(8,0.0,0.0,10.0);

  vtkCellArray* triangles=vtkCellArray::New();
  triangles->Allocate(10, 5);
  int p[3];
  for (int i=0;i<=7;i++)
    {
      p[0]=i;
      p[1]=i+1;
      if ( p[1] > 7)
	p[1]=0;
      p[2]=8;
	  
      triangles->InsertNextCell(3,p);
      fprintf(stdout,"Set Triangle %d = (%d,%d,%d)\n",i,p[0],p[1],p[2]);
    }

  vtkPolyData* surface=vtkPolyData::New();
  surface->SetPoints(pts);
  surface->SetPolys(triangles);


  // Since VTK uses reference counting delete pts and triangles
  // as these are counted inside surface
  pts->Delete();
  triangles->Delete();

  // Save the surface to a file
  vtkPolyDataWriter* writer=vtkPolyDataWriter::New();
  writer->SetInput(surface);
  writer->SetFileName("surface.vtk");
  writer->Write();

  // Clean Up all objects and exit
  writer->Delete();
  surface->Delete();

  return 0;
}
