#include "vtkImageExtractVOI.h"
#include "vtkObjectFactory.h"
#include "vtkImageData.h"
#include "vtkPointData.h"
#include "vtkImageReslice.h"
#include "vtkInformation.h"
#include "vtkInformationVector.h"
#include "vtkStreamingDemandDrivenPipeline.h"

vtkStandardNewMacro(vtkImageExtractVOI);


// Construct object to extract all of the input data.
vtkImageExtractVOI::vtkImageExtractVOI()
{
  this->VOI[0] = this->VOI[2] = this->VOI[4] = 0;
  this->VOI[1] = this->VOI[3] = this->VOI[5] = 1;
  this->SampleRate[0] = this->SampleRate[1] = this->SampleRate[2] = 1;
}

// ---------------------------------------------------------------------------

void vtkImageExtractVOI::PrintSelf(ostream& os, vtkIndent indent)
{
  vtkSimpleImageToImageFilter::PrintSelf(os,indent);

  os << indent << "  VOI " << this->VOI[0] << ":" << this->VOI[1] << " ";
  os << indent << "  VOI " << this->VOI[2] << ":" << this->VOI[3] << " ";
  os << indent << "  VOI " << this->VOI[4] << ":" << this->VOI[5] << "\n";
  os << indent << "  SampleRate "<<this->SampleRate[0]<< ","<<this->SampleRate[1]<<","<<this->SampleRate[2]<<"\n";
}
// ---------------------------------------------------------------------------
int vtkImageExtractVOI::RequestInformation(vtkInformation *request,  vtkInformationVector **inputVector, 
					   vtkInformationVector *outputVector)
{
  vtkInformation *inInfo =  inputVector[0]->GetInformationObject(0);
  vtkInformation *outInfo = outputVector->GetInformationObject(0);

  int wholeExtent[6]; inInfo->Get(vtkStreamingDemandDrivenPipeline::WHOLE_EXTENT(), wholeExtent);
  double sp[3];  inInfo->Get(vtkDataObject::SPACING(), sp);
  double origin[3];  inInfo->Get(vtkDataObject::ORIGIN(),  origin);

  vtkImageData* input=this->GetImageDataInput(0);
  input->GetExtent(wholeExtent);
  input->GetSpacing(sp);
  input->GetOrigin(origin);

  int nc=input->GetNumberOfScalarComponents();

  int voi[6],rate[3],outDims[3];
  for (int i=0; i < 6; i++ )
    voi[i] = this->VOI[i];
  
  for (int i=0; i < 3; i++ )
    {
      if ( voi[2*i+1] > wholeExtent[2*i+1] )
	{
	  voi[2*i+1] = wholeExtent[2*i+1];
	}
      else if ( voi[2*i+1] < wholeExtent[2*i] )
	{
	  voi[2*i+1] = wholeExtent[2*i];
	}
      if ( voi[2*i] < wholeExtent[2*i] )
	{
	  voi[2*i] = wholeExtent[2*i];
	}
      else if ( voi[2*i] > wholeExtent[2*i+1] )
	{
	  voi[2*i] = wholeExtent[2*i+1];
	}
      
      if ( voi[2*i] > voi[2*i+1] )
	{
	  voi[2*i] = voi[2*i+1];
	}
      
      if ( (rate[i] = this->SampleRate[i]) < 1 )
	{
	  rate[i] = 1;
	}
      
      outDims[i] = (voi[2*i+1] - voi[2*i]) / rate[i] + 1;
      if ( outDims[i] < 1 )
	{
	  outDims[i] = 1;
	}
    }
  
  // This makes sense for sample rates of 1, 1, 1.
  wholeExtent[0] = 0;
  wholeExtent[1] = outDims[0] - 1;
  wholeExtent[2] = 0;
  wholeExtent[3] = outDims[1] - 1;
  wholeExtent[4] = 0;
  wholeExtent[5] = outDims[2] - 1;
  
  outInfo->Set(vtkStreamingDemandDrivenPipeline::WHOLE_EXTENT(),wholeExtent,6);



  double osp[3],outori[3];
  for (int ia=0;ia<=2;ia++)
    {
      osp[ia]=sp[ia]*double(rate[ia]);
      outori[ia]=double(voi[ia])*sp[0]+origin[0];
    }

  outInfo->Set(vtkDataObject::SPACING(), osp, 3);
  outInfo->Set(vtkDataObject::ORIGIN(), outori, 3);

	return 1; 


}
// ---------------------------------------------------------------------------
void vtkImageExtractVOI::SimpleExecute(vtkImageData* input, vtkImageData* output) 
{
  if (input==NULL)
    {
      vtkErrorMacro(<< "Bad Input");
      return;
    }

  int dim[3]; output->GetDimensions(dim);


  vtkImageReslice* resl=vtkImageReslice::New();
  resl->SetInput(input);
  resl->SetOutputOrigin(output->GetOrigin());
  resl->SetOutputSpacing(output->GetSpacing());
  resl->SetInterpolationMode(0);
  resl->SetOutputExtent(0,dim[0]-1,0,dim[1]-1,0,dim[2]-1);
  resl->OptimizationOff();
  resl->Update();

  output->ShallowCopy(resl->GetOutput());
  resl->Delete();
}



