#include "vtkObjectFactory.h"
#include "vtkImageSimpleThreshold.h"
#include "vtkPointData.h"
#include "vtkDataArray.h"

//------------------------------------------------------------------------------
vtkStandardNewMacro(vtkImageSimpleThreshold);

//------------------------------------------------------------------------------
vtkImageSimpleThreshold::vtkImageSimpleThreshold()
{
  this->LowerThreshold=-1.0e+9;
  this->UpperThreshold= 1.0e+9;
  this->BinaryOutput=1;
}
// ----------------------------------------------------------------------------
void vtkImageSimpleThreshold::SimpleExecute(vtkImageData* input,vtkImageData* output )
{
  if (input==NULL)
    {
      vtkErrorMacro(<<"Bad Input to vtkImageSimpleThreshold");
      return;
    }

  int dim[3]; input->GetDimensions(dim);
  int numvox=dim[0]*dim[1]*dim[2];
  int numcomp=input->GetNumberOfScalarComponents();

  vtkDataArray* inarray=input->GetPointData()->GetScalars();
  vtkDataArray* outarray=output->GetPointData()->GetScalars();
  
  for (int component=0;component<numcomp;component++)
    {
      outarray->FillComponent(component,0.0);

      for (int i=0;i<numvox;i++)
	{
	  double out=0.0;
	  double v=inarray->GetComponent(i,component);
	  if (v>=this->LowerThreshold && v<=this->UpperThreshold)
	    {
	      if (this->BinaryOutput)
		out=1.0;
	      else
		out=v;
	      outarray->SetComponent(i,component,out);
	    }
	}
    }
}
// ----------------------------------------------------------------------------
