#include "vtkMyUtility.h"
#include "vtkObjectFactory.h"
#include <stdio.h>
#include "vtkImageData.h"
#include "vtkDataArray.h"
#include "vtkPointData.h"

// This creates the static member function New
vtkStandardNewMacro(vtkMyUtility);

// This function is accessible both from C++ and Tcl
double vtkMyUtility::Average(double a,double b)
{
  return 0.5*(a+b);
}

// This function is only accessible from C++
double vtkMyUtility::Average(double* a,int n)
{
  if (a==NULL)  return 0.0;
  double sum=0.0;
  for (int i=0;i<n;i++)
    sum+=a[i];
  return sum/double(n);
}


vtkImageData* vtkMyUtility::ThresholdImage(vtkImageData* input,double minv,double maxv,
					   int binary)
{
  if (input==NULL)
    {
      fprintf(stderr,"Bad Input Image\n");
      return NULL;
    }

  vtkImageData* output=vtkImageData::New();
  output->CopyStructure(input);
  output->AllocateScalars();

  vtkDataArray* inarray=input->GetPointData()->GetScalars();
  vtkDataArray* outarray=output->GetPointData()->GetScalars();

  int numcomp=inarray->GetNumberOfComponents();
  int numvox =inarray->GetNumberOfTuples();
  
  for (int component=0;component<numcomp;component++)
    {
      outarray->FillComponent(component,0.0);

      for (int i=0;i<numvox;i++)
	{
	  double out=0.0;
	  double v=inarray->GetComponent(i,component);
	  if (v>=minv && v<=maxv)
	    {
	      if (binary)
		out=1.0;
	      else
		out=v;
	      outarray->SetComponent(i,component,out);
	    }
	}
    }

  return output;
}

