package provide PointBasedRegistration 1.0

lappend auto_path [ file dirname [ info script ]]

package require vtk
package require vtkinteraction
package require Itcl
package require Iwidgets
package require SimpleSurfaceViewer
package require loadpointreg 

itcl::class PointBasedRegistration {

    protected variable referencesurface [ vtkPolyData New ]
    protected variable targetsurface [ vtkPolyData New ]
    protected variable currentregistration  0

    protected variable simplesurfaceviewer  0
    protected variable notebook             0
    private   common   thisparam
    protected variable basewidget           0
    protected variable outputbox            0
    protected variable outputmatrix         "1 0 0 0\n 0 1 0 0\n0 0 1 0\n0 0 0 1"

    constructor { base } {
	set basewidget $base
	$this Initialize
    }

    destructor  {
	$referencesurface Delete
	$targetsurface Delete
	catch [ $currentregistration Delete ]
	catch [ itcl::delete obj $simplesurfaceviewer  ]
    }


    public    method ComputeRegistration { }
    public    method RegistrationProgressUpdate  { }
    public    method LoadSurface { mode inputfilename }
    public    method SaveMatrix { }
    public    method ExitCommand { }

    # -----------------------------------------------

    protected method Initialize { }
    protected method ResetParameters  { }
    protected method CreateInputGUI   { par }
    protected method CreateComputeGUI { par }
    protected method CreateOutputGUI  { par }
    protected method CreateSurfaceViewer { par }
	    
};
# ---------------------------------------------------------------------------------------

itcl::body PointBasedRegistration::Initialize { } {

    $this ResetParameters

    toplevel $basewidget;    wm geometry $basewidget 800x400
    wm title $basewidget "Point Based Registration"
    update 

        
    set notebook  [ iwidgets::tabnotebook $basewidget.left  -tabpos n -width 250 ]
    set middle    [ frame $basewidget.middle -width 5 -bg black ]
    set viewframe [ frame $basewidget.right  -width 500 ]
    pack $notebook $middle -side left -expand false -fill y
    pack $viewframe -side right -expand true -fill both


    $this CreateInputGUI    [ $notebook add -label "Input" ]
    $this CreateComputeGUI  [ $notebook add -label "Compute" ]
    $this CreateOutputGUI   [ $notebook add -label "Output" ]
    $notebook view "Input"

    frame $viewframe.2;     pack $viewframe.2 -side bottom -expand false -fill x
    
    eval "button $viewframe.2.2 -text Exit -command { $this ExitCommand }"
    pack $viewframe.2.2 -side right -padx 1 -expand false
    iwidgets::entryfield $viewframe.2.1 -textvariable [ itcl::scope thisparam($this,icp_status) ] -width 50 -labeltext "Status:"
    pack $viewframe.2.1 -side left -padx 1 -expand true -fill x

    
    set simplesurfaceviewer [ CreateSurfaceViewer $viewframe.1 ]
    eval "wm protocol $basewidget WM_DELETE_WINDOW { $this ExitCommand }"
    
}

itcl::body PointBasedRegistration::CreateSurfaceViewer { par } {
    return [ [ SimpleSurfaceViewer \#auto $par ] GetThisPointer ]
}
# ---------------------------------------------------------------------------------------
itcl::body PointBasedRegistration::ResetParameters { } {

    set thisparam($this,referencename) "none.vtk"
    set thisparam($this,referencestatus) "No Surface Loaded"

    set thisparam($this,targetname) "none.vtk"
    set thisparam($this,targetstatus) "No Surface Loaded"
    
    set thisparam($this,icp_numberofpoints)     400
    set thisparam($this,icp_numberofiterations) 5
    set thisparam($this,icp_alignoriginsfirst)  1
    set thisparam($this,icp_transformationmode) Rigid
    set thisparam($this,icp_delay) 1
    set thisparam($this,icp_status) ""


}

# ---------------------------------------------------------------------------------------

itcl::body PointBasedRegistration::LoadSurface { mode inputfilename } {

    if { $mode == "Reference" } {
	set v1    referencename
	set v2   referencestatus
	set sur   $referencesurface
    } else {
	set v1    targetname
	set v2   targetstatus
	set sur   $targetsurface
    }

    set filename $thisparam($this,$v1)	
    if { [ string length $inputfilename ] < 1 } {
	set f2path [ file dirname $filename ]
	set f2tail [ file tail    $filename ]
	set fname  [tk_getOpenFile -title "Load $mode" -filetypes { {"Surface Files" {.vtk}}} -initialfile $f2tail -initialdir $f2path ]   
	if { [ string length $fname ] <1 } {
	    return 0
	}
    } else {
	set fname $inputfilename
    }

    set surfacereader [ vtkPolyDataReader New ]
    $surfacereader SetFileName $fname
    $surfacereader Update
    $sur ShallowCopy [ $surfacereader GetOutput ]
    $surfacereader Delete

    set thisparam($this,$v1) $fname
    set thisparam($this,$v2) "Number of Points: [ $sur GetNumberOfPoints ]"

    $simplesurfaceviewer SetSurfacesAndTransformation $referencesurface $targetsurface 0
    $simplesurfaceviewer ResetRenderer
}
# ---------------------------------------------------------------------------------------
itcl::body PointBasedRegistration::CreateInputGUI { base } {

    for { set i 0 } { $i <=1 } { incr i } {
	if { $i == 0 } {
	    set name "Reference"
	    set v1   referencename
	    set v2   referencestatus
	} else {
	    set name "Target"
	    set v1   targetname
	    set v2   targetstatus
	}

	iwidgets::Labeledframe $base.$i -labelpos nw -labeltext $name -relief ridge 
	pack $base.$i -side top -expand true -fill both -pady 2

	set w [ $base.$i childsite ]
	iwidgets::entryfield $w.1 -textvariable [ itcl::scope thisparam($this,$v1) ] -width 50 -labeltext "Filename:"
	frame $w.2
	pack $w.1 $w.2 -side top -expand t -fill x

	iwidgets::entryfield $w.2.1 -textvariable [ itcl::scope thisparam($this,$v2) ] -width 30 -labeltext "Status:"
	eval "button $w.2.2 -text Load -command { $this LoadSurface $name \"\" }"
	pack $w.2.2 -side right -expand false -padx 2
	pack $w.2.1 -side left  -expand true -fill x
    }
}
# ---------------------------------------------------------------------------------------
itcl::body PointBasedRegistration::CreateComputeGUI { base } {

    eval "button $base.bot -text \"Compute Registration\" -command { $this ComputeRegistration }"
    pack $base.bot -side bottom -expand false -fill x

    iwidgets::Labeledframe $base.top -labelpos nw -labeltext "Parameters" -relief ridge 
    pack $base.top -side top -expand true -fill both -pady 2

    set w [ $base.top childsite ]
    set k 0

    iwidgets::entryfield $w.$k -labeltext "Max Num Points:"  -width 5 -validate integer -textvariable [ itcl::scope thisparam($this,icp_numberofpoints) ] -relief sunken 
    pack $w.$k; incr k

    iwidgets::entryfield $w.$k -labeltext "Max Num Iterations:"  -width 5 -validate integer -textvariable [ itcl::scope thisparam($this,icp_numberofiterations) ] -relief sunken 
    pack $w.$k; incr k

    checkbutton $w.$k -variable [ itcl::scope thisparam($this,icp_alignoriginsfirst) ] -text "Align Origins First"
    pack $w.$k; incr k

    set cw [ [ iwidgets::labeledwidget  $w.$k -labeltext "Mode:"] childsite ];
    pack $w.$k; incr k
    tk_optionMenu $cw.optmenu [ itcl::scope thisparam($this,icp_transformationmode) ] "Rigid" "Similarity" "Affine"
    pack $cw.optmenu -side right -expand false -fill x

    iwidgets::entryfield $w.$k -labeltext "Delay (sec):"  -width 5 -validate integer -textvariable [ itcl::scope thisparam($this,icp_delay) ] -relief sunken 
    pack $w.$k; incr k

}
# ---------------------------------------------------------------------------------------
itcl::body PointBasedRegistration::CreateOutputGUI { base } {

    iwidgets::Labeledframe $base.top -labelpos nw -labeltext "Output Transformation" -relief ridge 
    pack $base.top -side top -expand true -fill both

    set w [ $base.top childsite ]

    eval "button $w.bot -text \"Save Matrix\" -command { $this SaveMatrix }"
    set outputbox [ text $w.top -width 35 -height 5 -borderwidth 2 -relief raised -font { fixed 8 }]
    pack $w.bot -side bottom -expand true -fill x
    pack $outputbox -side top -expand true -fill both

    $outputbox delete 1.0 end
    $outputbox insert end $outputmatrix
    $outputbox see 1.0
}
# ---------------------------------------------------------------------------------------
itcl::body PointBasedRegistration::SaveMatrix { } {

    set fname  [tk_getSaveFile -title "Saving Matrix" -filetypes { {"Matrix Files" {.matr}} } -initialfile "output.matr" ]
    if { [string length $fname ] > 0 } {
	set fileid [open $fname w]
	puts -nonewline $fileid $outputmatrix
	close $fileid
    }
}
# ---------------------------------------------------------------------------------------
itcl::body PointBasedRegistration::ComputeRegistration { } {

    set np1 [ $referencesurface GetNumberOfPoints ]
    set np2 [ $targetsurface GetNumberOfPoints ]
    if { $np1 < 10 || $np2 < 10 } {
	tk_messageBox -type ok -message "Not enough points to compute registration ($np1,$np2)" -title "Error!" -icon error
	return 0
    }


    $simplesurfaceviewer SetSurfacesAndTransformation $referencesurface $targetsurface 0
    $simplesurfaceviewer UpdateDisplay

    set icp [ vtkLinearICPRegistration New ]
    set currentregistration $icp
    $icp SetSource $referencesurface
    $icp SetTarget $targetsurface
    $icp SetMaximumNumberOfIterations $thisparam($this,icp_numberofiterations)
    $icp SetMaximumNumberOfPoints     $thisparam($this,icp_numberofpoints)
    $icp SetStartByMatchingCentroids  $thisparam($this,icp_alignoriginsfirst)

    if { $thisparam($this,icp_transformationmode) == "Rigid" } {
	$icp SetTransformationTypeToRigid
    } elseif {  $thisparam($this,icp_transformationmode) == "Similarity" } {
	$icp SetTransformationTypeToSimilarity
    } else {
	$icp SetTransformationTypeToAffine
    }
	    
    # Observer Stuff
    eval "$icp AddObserver ProgressEvent { $this RegistrationProgressUpdate }"
    eval "$icp AddObserver EndEvent      { $this  RegistrationProgressUpdate }"

    # Execute
    $icp Run

    set tr [ $icp GetTransformation ]
    set matr [ $tr GetMatrix ]

    set outputmatrix ""

    for { set i 0 } { $i < 4 } { incr i } {
	for { set j 0 } { $j < 4 } { incr j } {
	    set key [$matr GetElement $i $j]
	    set line [ format "%6.3f " $key ]
	    set outputmatrix "$outputmatrix $line"
	}
	set outputmatrix "$outputmatrix\n"
    }

    $outputbox delete 1.0 end
    $outputbox insert end $outputmatrix
    $outputbox see 1.0

    $icp Delete
    set currentregistration 0
    set thisparam($this,icp_status) "Done with Registration"
    $notebook view "Output"
}

# ---------------------------------------------------------------------------------------
itcl::body PointBasedRegistration::RegistrationProgressUpdate { } {
    $simplesurfaceviewer SetSurfacesAndTransformation $referencesurface $targetsurface [ $currentregistration GetTransformation ]
    $simplesurfaceviewer UpdateDisplay
    set thisparam($this,icp_status) [ format "Registration Progress %.2f " [ expr 100.0 * [ $currentregistration GetProgress ] ] ]
    update idletasks
    after [ expr 100 * $thisparam($this,icp_delay) ]
}
	
# ---------------------------------------------------------------------------------------
itcl::body PointBasedRegistration::ExitCommand { } {
    
#    vtkCommand DeleteAllObjects
    destroy .
    exit
}