package provide SimpleSurfaceViewer 1.0

lappend auto_path [ file dirname [ info script ]]
package require vtk
package require vtkinteraction
package require Itcl
package require Iwidgets

itcl::class SimpleSurfaceViewer {

    protected variable referencesurface [ vtkPolyData New ]
    protected variable transformfilter  [ vtkTransformPolyDataFilter New ]
    protected variable targetsurface    [ vtkPolyData New ]
    protected variable transformation   0
    
    protected variable referenceactor    [ vtkActor New ]
    protected variable targetactor    [ vtkActor New ]

    protected variable renderwidget   0
    protected variable renderer       0

    protected variable basewidget  0
    private   common   thisparam  
    
    constructor { base }  {
	set basewidget $base
	set thisparam($this,referencemode) "Wireframe"
	set thisparam($this,targetmode) "Wireframe"

	$this Initialize 

    }

    destructor  {
	$referencesurface Delete
	$transformfilter  Delete
	$targetsurface Delete
	$referenceactor Delete
	$targetactor Delete

	$renderer Delete
	$renderwidget Delete
    }
	

    public method SetSurfacesAndTransformation { reference target transform }
    public method SetSurfaceMode { md }
    public method ResetRenderer { }
    public method UpdateDisplay { }
    public method GetThisPointer { } { return $this }
    # ------------------------------------------------------------

    protected method Initialize { }
    protected method CreatePipelines { }
    protected method AddInteractor { renderwidget renwin   }
};

# --------------------------------------------------------------------------------------
itcl::body SimpleSurfaceViewer::CreatePipelines { } {

    # Create a Shpere
    set spheresource [ vtkSphereSource   New]
    $spheresource Update

    $referencesurface ShallowCopy [ $spheresource GetOutput ]
    $targetsurface    ShallowCopy [ $spheresource GetOutput ]


    # Reference Side
    $transformfilter SetInput $referencesurface
    set temp  [ vtkIdentityTransform New ]
    $transformfilter SetTransform $temp
    $temp Delete

    set map1 [ vtkPolyDataMapper New]
    $map1 SetInput [ $transformfilter GetOutput ] 
    $referenceactor SetMapper $map1; $map1 Delete
    
    # Target Side
    set map2 [ vtkPolyDataMapper New]
    $map2 SetInput $targetsurface
    $targetactor SetMapper $map2; $map2 Delete

    set actlist [ list $referenceactor $targetactor ]
    for { set i 0 } { $i <=1 } { incr i } {
	set act [ lindex $actlist $i ]

	[ $act GetProperty] SetRepresentationToWireframe
	[ $act GetProperty ] SetAmbient 1.0 
	[ $act GetProperty ] SetDiffuse 0.0
	[ $act GetProperty ] SetOpacity 1.0
	if { $i == 1 } {
	    [ $act GetProperty] SetColor 1.0 0.0 0.0
	} else {
	    [ $act GetProperty] SetColor 0.0 1.0 0.0
	}
    }

    $spheresource Delete

}
# --------------------------------------------------------------------------------------
itcl::body SimpleSurfaceViewer::Initialize { } {

    iwidgets::Labeledframe $basewidget -labelpos nw -labeltext "Surface Viewer" -relief ridge 
    pack $basewidget -side top -expand true -fill both

    set labelframe [ $basewidget childsite ]

    set buttonbar [ frame $labelframe.a1 ]
    pack $buttonbar -side bottom -expand false -fill x -pady 2


    # Create the Viewer First
    # -----------------------
    $this CreatePipelines
    
    set renderwidget [ vtkTkRenderWidget $labelframe.a2 ]
    pack $renderwidget -side top -expand true -fill both
    
    set renwin [ $renderwidget GetRenderWindow ]
    set renderer [ vtkRenderer New]
    $renwin AddRenderer $renderer
    
    $renderer AddActor $referenceactor
    $renderer AddActor $targetactor


    # Create the Buttons Next
    # -----------------------
    eval "button $buttonbar.reset -text Reset -command { $this ResetRenderer}"
    pack $buttonbar.reset -side right -padx 2

    label $buttonbar.m -text "Reference:"
    tk_optionMenu $buttonbar.opt   [ itcl::scope thisparam($this,referencemode) ] "Surface" "Wireframe" "Points" "None"
    label $buttonbar.m2 -text "Target:"
    tk_optionMenu $buttonbar.opt2  [ itcl::scope thisparam($this,targetmode) ] "Surface" "Wireframe" "Points" "None"
    pack $buttonbar.m $buttonbar.opt $buttonbar.m2 $buttonbar.opt2 -side left -fill x -expand t
    
    for { set k 0 } { $k <=2 } { incr k } {
	eval "$buttonbar.opt.menu   entryconfigure $k -command { $this SetSurfaceMode 1 }"
	eval "$buttonbar.opt2.menu  entryconfigure $k -command { $this SetSurfaceMode 2 }"
    }

    $this AddInteractor $renderwidget $renwin
}

itcl::body SimpleSurfaceViewer::AddInteractor { renderwidget renwin } {
    
    ::vtk::bind_tk_widget $renderwidget $renwin
    update idletasks
    [ $renwin GetInteractor ] ExposeEvent

}
    
# --------------------------------------------------------------------------------------
itcl::body SimpleSurfaceViewer::SetSurfacesAndTransformation { reference target transformation } {

    $referencesurface ShallowCopy $reference

    if { $transformation != 0 } {
	$transformfilter  SetTransform $transformation; $transformfilter Update
    } else {
	set temp  [ vtkIdentityTransform New ];
	$transformfilter SetTransform $temp
	$temp Delete
    }
    $targetsurface    ShallowCopy $target

    
}
# --------------------------------------------------------------------------------------
itcl::body SimpleSurfaceViewer::SetSurfaceMode { md } {

    if { $md == 1 } {
	set act  $referenceactor
	set mode $thisparam($this,referencemode)
    } else {
	set act $targetactor
	set mode $thisparam($this,targetmode)
    }
    

    if { $mode != "None" } {
	$act SetVisibility 1
	[ $act GetProperty ] SetRepresentationTo$mode
	if { [  [ $act GetProperty ] GetRepresentationAsString ] == "Surface" } {
	    [ $act GetProperty ] SetAmbient 0.0 
	    [ $act GetProperty ] SetDiffuse 1.0
	    [ $act GetProperty ] SetOpacity 1.0
	} else {
	    [ $act GetProperty ] SetAmbient 1.0 
	    [ $act GetProperty ] SetDiffuse 0.0
	    [ $act GetProperty ] SetOpacity 1.0
	    [ $act GetProperty ] SetLineWidth 1.0
	    [ $act GetProperty ] SetPointSize 3.0
	}
    } else {
	$act SetVisibility 0
    }
    
    $this UpdateDisplay
}

# --------------------------------------------------------------------------------------
itcl::body SimpleSurfaceViewer::ResetRenderer { } {
    $renderer ResetCamera
    $renderer ResetCameraClippingRange
    $this UpdateDisplay
}

# --------------------------------------------------------------------------------------
itcl::body SimpleSurfaceViewer::UpdateDisplay { } {
    [ $renderer GetRenderWindow ] Render 
}
# --------------------------------------------------------------------------------------
