#include "vtkAbstractPointBasedRegistration.h"
#include "vtkDataSet.h"
#include "vtkObjectFactory.h"
#include "vtkPoints.h"

vtkCxxRevisionMacro(vtkAbstractPointBasedRegistration, "$Revision: 1.0 $");

//----------------------------------------------------------------------------

vtkAbstractPointBasedRegistration::vtkAbstractPointBasedRegistration() : vtkProcessObject()
{
  this->Source = NULL;
  this->Target = NULL;
  this->MaximumNumberOfIterations = 50;
  this->MaximumNumberOfPoints = 200;
  this->Epsilon=0.001;
}

//----------------------------------------------------------------------------
vtkAbstractPointBasedRegistration::~vtkAbstractPointBasedRegistration()
{
  this->SetSource(NULL);
  this->SetTarget(NULL);
}

//----------------------------------------------------------------------------
void vtkAbstractPointBasedRegistration::PrintSelf(ostream& os, vtkIndent indent)
{
  this->Superclass::PrintSelf(os,indent);

  if ( this->Source ) 
    os << indent << "Source: " << this->Source << "\n";
  else 
    os << indent << "Source: (none)\n";


  if ( this->Target ) 
    os << indent << "Target: " << this->Target << "\n";
  else 
    os << indent << "Target: (none)\n";


  os << indent << "MaximumNumberOfIterations: " << this->MaximumNumberOfIterations << "\n";
  os << indent << "MaximumNumberOfPoints: " << this->MaximumNumberOfPoints << "\n";
  os << indent << "Epsilon: " << this->Epsilon << "\n";

}
//----------------------------------------------------------------------------
vtkPoints* vtkAbstractPointBasedRegistration::SampleDataSet(vtkDataSet* input,int NumberOfPoints)
{
  if (input==NULL)
    {
      vtkErrorMacro(<<"Bad Input to Sample Data Set\n");
      return NULL;
    }

  int np=input->GetNumberOfPoints();

  if (NumberOfPoints<1)
    NumberOfPoints=np;

  
  int samplerate=np/NumberOfPoints;
  if (samplerate<1)
    samplerate=1;
  int outp=np/samplerate;

  vtkDebugMacro(<<"Sampling Input Data Set from "<< np << " points to " << outp << "points. Samplerate="<<samplerate<<"\n");

  vtkPoints* OutputPoints=vtkPoints::New();
  OutputPoints->SetNumberOfPoints(outp);
  int index=0;
  for (int i=0;i<outp;i++)
    {
      OutputPoints->SetPoint(i,input->GetPoint(index));
      index+=samplerate;
    }

  return OutputPoints;
}

//-----------------------------------------------------------------------------------------
void vtkAbstractPointBasedRegistration::GetCentroid(vtkDataSet* input,double centroid[3])
{
  centroid[0]=0.0;      centroid[1]=0.0;      centroid[2]=0.0;

  if (input==NULL)
    {
      vtkErrorMacro(<<"Bad Input to GetCentroid\n");
      return;
    }

  int np=input->GetNumberOfPoints();
  if (np<1)
    {
      vtkErrorMacro(<<"Bad Input (no points) to GetCentroid\n");
      return;
    }

  double sum[3]={0.0,0.0,0.0};


  for (int i=0;i<np;i++)
    {
      double x[3]; input->GetPoint(i,x);
      for (int ia=0;ia<=2;ia++)
	sum[ia]+=x[ia];
    }

  for (int ia=0;ia<=2;ia++)
    centroid[ia]=sum[ia]/double(np);
}
//-----------------------------------------------------------------------------------------
void vtkAbstractPointBasedRegistration::ShiftPoints(vtkPoints* input,double shift[3],double scale)
{
  if (input==NULL)
    {
      vtkErrorMacro(<<"Bad Input to Shift Points\n");
      return;
    }
  
  int np=input->GetNumberOfPoints();
  for (int i=0;i<np;i++)
    {
      double x[3]; input->GetPoint(i,x);
      for (int ia=0;ia<=2;ia++)
	x[ia]=x[ia]+shift[ia]*scale;
      input->SetPoint(i,x);
    }
}
//-----------------------------------------------------------------------------------------
