/*=========================================================================

This code is partially adapted from the class vtkIterativeClosestPointTransform
in VTK 4.4


=========================================================================*/


#ifndef __vtkAbstractPointBasedRegistration_h
#define __vtkAbstractPointBasedRegistration_h

#include "vtkProcessObject.h"
#include "vtkDataSet.h"
#include "vtkAbstractTransform.h"
#include "vtkPoints.h"

class vtkAbstractPointBasedRegistration : public vtkProcessObject
{
public:
  vtkTypeRevisionMacro(vtkAbstractPointBasedRegistration,vtkProcessObject);
  void PrintSelf(ostream& os, vtkIndent indent);

  // Description:
  // Specify the source and target data sets.
  vtkSetObjectMacro(Source, vtkDataSet);
  vtkSetObjectMacro(Target, vtkDataSet);

  vtkGetObjectMacro(Source, vtkDataSet);
  vtkGetObjectMacro(Target, vtkDataSet);

  // Description: 
  // Set/Get the  maximum number of iterations
  vtkSetMacro(MaximumNumberOfIterations, int);
  vtkGetMacro(MaximumNumberOfIterations, int);


  // Description: 
  // Set/Get the maximum number of landmarks sampled in your dataset.
  // If your dataset is dense, then you will typically not need all the 
  // points to compute the ICP transform. 
  vtkSetMacro(MaximumNumberOfPoints, int);
  vtkGetMacro(MaximumNumberOfPoints, int);

  // Description: 
  // Set/Get the convergence factor
  vtkSetMacro(Epsilon, double);
  vtkGetMacro(Epsilon, double);


  // Description: 
  // Get the output transformation
  // This is purposefully an abstract transformation so that derived classes can
  // return different types of transformations
  virtual vtkAbstractTransform* GetTransformation() = 0;
  
  // Description:
  // The Run Method Computes the Registration
  // This is an abstract class, so in this case Run is a pure virtual method
  virtual int Run() = 0;
  
protected:


  vtkAbstractPointBasedRegistration();
  virtual ~vtkAbstractPointBasedRegistration();

  // Description:
  // Data Members
  vtkDataSet* Source;
  vtkDataSet* Target;
  int MaximumNumberOfIterations;
  int MaximumNumberOfPoints;
  double Epsilon;

  // Description:
  // Utility Method
  virtual   vtkPoints* SampleDataSet(vtkDataSet* input,int NumberOfPoints);
  virtual   void       GetCentroid(vtkDataSet* input,double centroid[3]);
  virtual   void       ShiftPoints(vtkPoints* input,double shift[3],double scale=1.0);

private:
  vtkAbstractPointBasedRegistration(const vtkAbstractPointBasedRegistration&);  // Not implemented.
  void operator=(const vtkAbstractPointBasedRegistration&);  // Not implemented.
};

#endif
