/*=========================================================================

This code is partially adapted from the class vtkIterativeClosestPointTransform
in VTK 4.4


=========================================================================*/


#ifndef __vtkLinearICPRegistration_h
#define __vtkLinearICPRegistration_h

#include "vtkAbstractPointBasedRegistration.h"
#include "vtkTransform.h"
#include "vtkDataSet.h"

class vtkLinearICPRegistration : public vtkAbstractPointBasedRegistration
{
public:
  static vtkLinearICPRegistration *New();
  vtkTypeRevisionMacro(vtkLinearICPRegistration,vtkAbstractPointBasedRegistration);
  void PrintSelf(ostream& os, vtkIndent indent);

  // Description: 
  // Starts the process by translating source centroid to target centroid.
  vtkSetMacro(StartByMatchingCentroids, int);
  vtkGetMacro(StartByMatchingCentroids, int);
  vtkBooleanMacro(StartByMatchingCentroids, int);

  // Description:
  // Transformation Type, Rigid, Similarity, Affine
  vtkGetMacro(TransformationType,int);
  vtkSetClampMacro(TransformationType,int,0,2);
  virtual void SetTransformationTypeToRigid() { this->SetTransformationType(0);}
  virtual void SetTransformationTypeToSimilarity() { this->SetTransformationType(1);}
  virtual void SetTransformationTypeToAffine() { this->SetTransformationType(2);}

  // Description: 
  // Get the output transformation
  // This is purposefully an abstract transformation so that derived classes can
  // return different types of transformations
  virtual vtkAbstractTransform* GetTransformation();
  
  // Description:
  // The Run Method Computes the Registration
  // This is an abstract class, so in this case Run is a pure virtual method
  virtual int Run();


  
protected:
  vtkLinearICPRegistration();
  virtual ~vtkLinearICPRegistration();

  // Description:
  // Data Members
  int StartByMatchingCentroids;
  int TransformationType;
  vtkTransform          *OutputTransformation;


private:
  vtkLinearICPRegistration(const vtkLinearICPRegistration&);  // Not implemented.
  void operator=(const vtkLinearICPRegistration&);  // Not implemented.
};

#endif
