#ifndef __vtkKMeansSegmentation_h
#define __vtkKMeansSegmentation_h

#include "vtkSimpleImageToImageFilter.h"
#include "vtkImageData.h"
#include "vtkDoubleArray.h"


class vtkKMeansSegmentation : public vtkSimpleImageToImageFilter
{
public:
  static vtkKMeansSegmentation *New();
  vtkTypeMacro(vtkKMeansSegmentation,vtkSimpleImageToImageFilter);

  // Description:
  // Sigma Ratio constrains the ratio of the standard deviations
  vtkGetMacro(MaxSigmaRatio,double);
  vtkSetClampMacro(MaxSigmaRatio,double,0.0001,1.0);

  // Description:
  // Number Of Classes
  vtkGetMacro(NumberOfClasses,int);
  vtkSetClampMacro(NumberOfClasses,int,1,10);

  // Description:
  // Number Of Iterations
  vtkGetMacro(Iterations,int);
  vtkSetClampMacro(Iterations,int,0,200);

  // Description:
  // Number Of Bins for Histogram
  vtkGetMacro(NumberOfBins,int);
  vtkSetClampMacro(NumberOfBins,int,4,1024);

  // Description:
  // Convergence for KMeans Values (<1.0)
  vtkGetMacro(Convergence,double);
  vtkSetClampMacro(Convergence,double,0.05,100);

  // Description:
  // Get The Histogram and The Parameters if desired
  vtkGetObjectMacro(Histogram,vtkImageData);
  vtkGetObjectMacro(Parameters,vtkDoubleArray);

protected:

  vtkKMeansSegmentation();
  virtual ~vtkKMeansSegmentation();
  vtkKMeansSegmentation(const vtkKMeansSegmentation&) {};
  void operator=(const vtkKMeansSegmentation&) {};

  virtual int RequestInformation(vtkInformation *vtkNotUsed(request),  vtkInformationVector **inputVector, 
				 vtkInformationVector *outputVector);
  virtual void SimpleExecute(vtkImageData* in,vtkImageData* out);

  // Description:
  // Metric
  virtual double Metric(double x,double m,double sigma2);


  // Description:
  // Components 
  virtual vtkImageData* CreateHistogram(vtkImageData* input,int NumBins);
  virtual int InitializeParameters(vtkImageData* histogram,int numclasses,vtkDoubleArray* params);

  int             NumberOfClasses;
  int             Iterations;
  double          Convergence;
  double          HistogramOrigin;
  double          HistogramSpacing;
  vtkDoubleArray* Parameters;
  vtkImageData*   Histogram;
  int             NumberOfBins;
  double          MaxSigmaRatio;

};

#endif
