#include "vtkObjectFactory.h"
#include "vtkMultiThresholdSegmentation.h"
#include "vtkDoubleArray.h"
#include "vtkImageAccumulate.h"
#include "vtkMath.h"
#include "vtkPointData.h"
#include "vtkInformation.h"
#include "vtkInformationVector.h"
#include "vtkStreamingDemandDrivenPipeline.h"

//------------------------------------------------------------------------------

vtkMultiThresholdSegmentation* vtkMultiThresholdSegmentation::New()
{
  // First try to create the object from the vtkObjectFactory
  vtkObject* ret = vtkObjectFactory::CreateInstance("vtkMultiThresholdSegmentation");
  if(ret)
    {
      return (vtkMultiThresholdSegmentation*)ret;
    }
  // If the factory was unable to create the object, then create it here.
  return new vtkMultiThresholdSegmentation;
}

// Construct object with no children.
vtkMultiThresholdSegmentation::vtkMultiThresholdSegmentation()
{
  this->Thresholds=NULL;
}
// ----------------------------------------------------------------------------
vtkMultiThresholdSegmentation::~vtkMultiThresholdSegmentation()
{
  this->SetThresholds(NULL);
}
// ---------------------------------------------------------------------------------------------
//
//     Execute Segmentation 
//
// ----------------------------------------------------------------------------
int vtkMultiThresholdSegmentation::RequestInformation(vtkInformation *request,  vtkInformationVector **inputVector, 
					   vtkInformationVector *outputVector)
{
	vtkInformation *outInfo = outputVector->GetInformationObject(0);
	vtkDataObject::SetPointDataActiveScalarInfo(outputVector->GetInformationObject(0), -1, 1);
	vtkDataObject::SetPointDataActiveScalarInfo(outInfo, VTK_SHORT, -1);


  return 1; 
}

void vtkMultiThresholdSegmentation::SimpleExecute(vtkImageData* input,vtkImageData* output )
{
  if (input==NULL || this->Thresholds==NULL)
    {
      vtkErrorMacro(<<"Bad Inputs to vtkMultiThresholdSegmentation");
      return;
    }
  
  int num=this->Thresholds->GetNumberOfTuples();
  int NumberOfClasses=num+1;
  double* thr=new double[num];
  double* outthr=new double[num];
  for (int ia=0;ia<num;ia++)
      thr[ia]=this->Thresholds->GetComponent(ia,0);


  for (int c=0;c<num;c++)
    {
      int best=c;
      double bestt=1.1e+10;
      for (int c1=0;c1<num;c1++)
	{
	  if (thr[c1]<bestt)
	    {
	      bestt=thr[c1];
	      best=c1;
	    }
	}
      
      outthr[c]=thr[best];
      thr[best]=1e+10;
    }

  vtkDataArray* inpdata=input->GetPointData()->GetScalars();
  vtkDataArray* outdata=output->GetPointData()->GetScalars();
  outdata->FillComponent(0,0.0);
  int numvoxels=inpdata->GetNumberOfTuples();

  for (int voxel=0;voxel<numvoxels;voxel++)
    {
      int c=0;
      double v=inpdata->GetComponent(voxel,0);
      int done=0;
      while (c<num && done==0)
	{
	  if (v<outthr[c])
	    {
	      outdata->SetComponent(voxel,0,c);
	      done=1;
	    }
	  else
	    c++;
	}

      if (done==0)
	outdata->SetComponent(voxel,0,NumberOfClasses-1);
    }
  delete [] thr;
  delete [] outthr;
}

