#include "vtkMyUtility.h"
#include "vtkObjectFactory.h"
#include <stdio.h>
#include "vtkImageData.h"
#include "vtkDataArray.h"
#include "vtkPointData.h"
#include "vtkPolyData.h"
#include "vtkContourFilter.h"
#include "vtkImageGradient.h"
#include "vtkImageGaussianSmooth.h"


// This creates the static member function New
vtkStandardNewMacro(vtkMyUtility);

vtkPolyData* vtkMyUtility::ExtractContour(vtkImageData* img,double level)
{
  if (img == NULL)
    {
      fprintf(stderr,"Bad Input to Extract Contour\n");
      return NULL;
    }

  vtkContourFilter *ContourFilter = vtkContourFilter::New();
  ContourFilter->SetInput(img);
  ContourFilter->SetValue(1, level);
  ContourFilter->Update();

  vtkPolyData* zerosurface=vtkPolyData::New();
  zerosurface->ShallowCopy(ContourFilter->GetOutput());
  ContourFilter->Delete();

  return zerosurface;
}

vtkImageData* vtkMyUtility::SmoothImageAndComputeGradient(vtkImageData* input,
							  double sigma,int dimensionality)
{
  if (input == NULL)
    {
      fprintf(stderr,"Bad Input to SmoothImageAndComputeGradient\n");
      return NULL;
    }


  vtkImageGaussianSmooth* sm=vtkImageGaussianSmooth::New();
  sm->SetInput(input);
  sm->SetStandardDeviations(sigma,sigma,sigma);
  sm->SetDimensionality(dimensionality);

  vtkImageGradient* gradient=vtkImageGradient::New();
  gradient->SetInput(sm->GetOutput());
  sm->Delete();
  gradient->SetDimensionality(dimensionality);
  gradient->Update();

  vtkImageData* grad=vtkImageData::New();
  grad->ShallowCopy(gradient->GetOutput());
  gradient->Delete();
  return grad;
}
