#include "vtkImageImport.h"
#include "vtkImageExport.h"
#include "vtkitkSignedDanielssonDistanceMapImageFilter.h"
#include "itkSignedDanielssonDistanceMapImageFilter.h"
#include "vtkImageThreshold.h"
#include "vtkInformationVector.h"
#include "vtkStreamingDemandDrivenPipeline.h"
#include "vtkInformation.h"
#include "itkImage.h"
#include "itkVTKImageExport.h"
#include "itkVTKImageImport.h"
#include "vtkITKUtility.h"



//------------------------------------------------------------------------------
// First a Helper Class
// -----------------------------------------------------------------------------
vtkitkSignedDanielssonDistanceMapImageFilter* vtkitkSignedDanielssonDistanceMapImageFilter::New()
{
  return new vtkitkSignedDanielssonDistanceMapImageFilter;
}

vtkitkSignedDanielssonDistanceMapImageFilter::vtkitkSignedDanielssonDistanceMapImageFilter()
{
  this->LowerThreshold=100;
  this->UpperThreshold=2000;
  this->InValue=3000;
  this->OutValue=0;
}
// ----------------------------------------------------------------------------
vtkitkSignedDanielssonDistanceMapImageFilter::~vtkitkSignedDanielssonDistanceMapImageFilter()
{
}
// ---------------------------------------------------------------------------------------------
template <int ImageDimension>
int vtkitkSignedDanielssonDistanceMapImageFilter_RunFilter(vtkImageData* input,vtkImageData* output,vtkitkSignedDanielssonDistanceMapImageFilter* self)
{
  // Step 1: Create Type definitions
  typedef float  PixelType;
  typedef itk::Image<PixelType,ImageDimension>    ImageType;
  typedef itk::VTKImageImport<ImageType> ImportType;
  typedef itk::VTKImageExport<ImageType> ExportType;
  typedef itk::SignedDanielssonDistanceMapImageFilter<ImageType,ImageType>  DMapFilterType; 

  // Step 2: Take VTK Image and make it float
  vtkImageThreshold* thresholder=vtkImageThreshold::New();
  thresholder->SetInput(input);
  thresholder->SetOutputScalarTypeToShort();
  thresholder->SetOutValue(self->GetOutValue());
  thresholder->SetInValue(self->GetInValue());
  thresholder->ThresholdBetween(self->GetLowerThreshold(),self->GetUpperThreshold());
  thresholder->SetOutputScalarTypeToFloat();
  thresholder->ReplaceInOn();
  thresholder->ReplaceOutOn();

  // Step 3: Convert VTK Image to ITK
  vtkImageExport *vtkInputExporter = vtkImageExport::New();
  vtkInputExporter->SetInput(thresholder->GetOutput());
  typename ImportType::Pointer itkInputImporter =  ImportType::New();
  ConnectPipelines(vtkInputExporter, itkInputImporter);
  itkInputImporter->Update();

  // Step 4: Run the ITK Code
  typename  DMapFilterType::Pointer dmap =    DMapFilterType::New();
  dmap->SetInput(itkInputImporter->GetOutput());
  dmap->Update();


  // Step 5: Push it out to VTK
  typename ExportType::Pointer itkExporter = ExportType::New();
  itkExporter->SetInput(dmap->GetOutput());
  itkExporter->Update();

  vtkImageImport* vtkImporter=vtkImageImport::New();
  ConnectPipelines(itkExporter, vtkImporter);
  vtkImporter->Update();
  output->DeepCopy(vtkImporter->GetOutput());

  // Step 6: Cleanup VTK filters only
  vtkImporter->Delete();
  thresholder->Delete();
  vtkInputExporter->Delete();
  return 1;
}


int vtkitkSignedDanielssonDistanceMapImageFilter::RunFilter(vtkImageData* input,vtkImageData* output)
{
  int dim[3]; input->GetDimensions(dim);


  if (dim[2]==1)
    return vtkitkSignedDanielssonDistanceMapImageFilter_RunFilter<2>(input,output,this);

  return vtkitkSignedDanielssonDistanceMapImageFilter_RunFilter<3>(input,output,this);
}
// ----------------------------------------------------------------------------
void vtkitkSignedDanielssonDistanceMapImageFilter::SimpleExecute(vtkImageData* input,vtkImageData* output)
{
  if (input==NULL)
    {
      vtkErrorMacro(<<"Bad Inputs to vtkitkSignedDanielssonDistanceMapImageFilter");
      return;
    }

  this->RunFilter(input,output);
}


// Tell the pipeline that the output has same dimensions as input but force float output
// --------------------------------------------------------------------------------------------
int vtkitkSignedDanielssonDistanceMapImageFilter::RequestInformation(vtkInformation *request,  vtkInformationVector **inputVector, 
								     vtkInformationVector *outputVector)
{
  vtkDataObject::SetPointDataActiveScalarInfo(outputVector->GetInformationObject(0), VTK_FLOAT, 1);
  return 1;
}
