#!/bin/sh
# the next line is executed by the shell, but it is a comment in tcl \
  exec wish "$0" "$@"


proc InitializeParameters { } {
    
    global parameters
    set parameters(enabletimestamp) 0
    set parameters(enableediting) 1
}

proc CreateScrolledText { basewidget } {
    
    global parameters

    set w [ labelframe $basewidget.log -text "Text File"  ]
    pack $w -side top -fill both -expand t
    
    eval "text $w.log -width 80 -height 10  -borderwidth 2 -relief raised -setgrid true  -yscrollcommand {$w.scroll set}"
    
    set cmd "scrollbar $w.scroll -command { $w.log yview} "
    eval $cmd
    
    pack $w.scroll -side right -fill y
    pack $w.log -side left -fill both -expand true


    return $w.log
}

proc ClearAllText { } {
    global parameters

    set numlines [ expr int([ $parameters(textbox) index end ]) -1 ]
    if { $numlines > 1 } {
	set ok [  tk_messageBox -type yesno -default no -title "Think again ..." -message "This will erase your current file. Are you sure?" -icon question  ]
	if { $ok == "no" } {
	    return 0
	}
    }

    $parameters(textbox) delete 1.0 end
    return 1
}

proc AddANewLine { } {
    global parameters

    set line "Some More Text" 
    if { $parameters(enabletimestamp) } {
	set currenttime [  clock format [ clock seconds ] -format "%H:%M:%S on %d %b %Y" ]
	set line "$line at $currenttime"
    }
    $parameters(textbox) insert end "$line\n"
    $parameters(textbox) see end
}

proc SetEditableState { } {
    global parameters

    if { $parameters(enableediting) == 1 } {
	$parameters(textbox) configure -state normal
    } else {
	$parameters(textbox) configure -state disabled
    }
}

proc Load { } {

    global parameters

    set ok [ ClearAllText ]
    if { $ok == 0 } {
	return 0
    }

    set fname [tk_getOpenFile -title "Load Filename" -filetypes { { "Text Files" {.tex .txt } }} ]
    if { [ string length $fname ]  < 1 } {
	return
    }

    set fileid 0
    catch { set fileid [open $fname r] }
    if { $fileid == 0 } {
	tk_messageBox -type ok  -title "Error ..." -message "Cannot read file $fname" -icon error  
	return
    }


    while { [ gets $fileid line ] >=0 } {
	$parameters(textbox) insert end "$line\n"
    }
    close $fileid
    wm title . "Simple Text Editor [ file tail $fname]"
}

proc Save { } {
    global parameters
    
    set numlines [ expr int([ $parameters(textbox) index end ]) -1 ]

    set fname [tk_getSaveFile -title "Save Filename" -filetypes { { "Text Files" {.tex .txt } }} ]
    if { [ string length $fname ]  < 1 } {
	return
    }

    set fileid 0
    catch { set fileid [open $fname w] } 

    if { $fileid == 0 } {
	tk_messageBox -type ok -title "Error ..." -message "Cannot save to file $fname" -icon error  ]
	return
    }

    for { set i 1 } { $i < $numlines } { incr i } {
	set ip [ expr $i + 1 ]
	set txt  [ $parameters(textbox) get $i.0 $ip.0 ] 
	puts -nonewline $fileid  "$txt"
    }
    close $fileid
    wm title . "Simple Text Editor [ file tail $fname]"
}


proc GenerateGUI { } { 
		   
    global parameters

    set menubase [ menu .menu ]
    . configure -menu $menubase
    
    frame .top; pack .top -side top -expand true -fill both
    
    set filem [ menu $menubase.file -tearoff 0 ]
    set filee [ menu $menubase.edit -tearoff 0 ]
    $menubase add cascade -underline 0 -label "File" -menu $filem
    $menubase add cascade -underline 0 -label "Edit" -menu $filee
    
    $filem add command -label "Clear Text" -command { ClearAllText  }
    $filem add separator
    $filem add command -label "Load" -command { Load  }
    $filem add command -label "Save" -command { Save  }
    $filem add separator
    $filem add command -label "Exit" -command { exit  }    
    
    $filee add check   -label "Enable TimeStamp" -variable parameters(enabletimestamp)
    $filee add check   -label "Enable Editing"   -variable parameters(enableediting) -command { SetEditableState }
    $filee add separator
    
    $filee add command -label "Add A New Line" -command { AddANewLine  }
    
    set parameters(textbox) [ CreateScrolledText .top ]
}

InitializeParameters
GenerateGUI
wm title . "Simple Text Editor"
