#!/bin/sh
# the next line is executed by the shell, but it is a comment in tcl \
  exec wish "$0" "$@"

package require Itcl
package require Iwidgets

itcl::class myGUI {

    private common parameters
    protected variable basewidget 0

    constructor { parent } { CreateGUI $parent }
    destructor { } 

    public method Reset { ask }
    public method SetColor { widget }
    public method GetName { mode }
    public method CreateGUI { base } 
};


itcl::body myGUI::Reset { ask } {

    if { $ask == 1 } {
	set ok [  tk_messageBox -type yesno -default no -title "Think again ..." -message "Reseting filenames" -icon question  ]
	if { $ok == "no"  } {
	    return
	}
    }
    set parameters($this,readname)  ""
    set parameters($this,writename) ""
    set parameters($this,directory) ""
}

itcl::body myGUI::SetColor { widget  } {
    set color  [ tk_chooseColor -title "Set Background Color"  -parent $basewidget ]
    if { [ string length $color ] > 0 } {
	$widget configure -bg $color
    }
}

itcl::body myGUI::GetName { mode } {

    set fname ""
    set filetype1 [ list "Text Files" [ list .txt .tex ] ]
    set filetype2 [ list "All Files" "*"  ]

    switch -exact $mode {
	"readname" {
	    set fname [tk_getOpenFile -title "Load Filename" -filetypes [ list $filetype1 $filetype2 ] ]
	}
	"writename" {	  
	    set fname [tk_getSaveFile -title "Filename" -defaultextension .txt -initialfile default.txt  -filetypes [ list $filetype1 ] ]
	}
	"directory" {
	    set fname  [tk_chooseDirectory -title "Select Current Directory" ]
	}
    }

    if { [ string length $fname ] < 1 } {
	return
    }

    set parameters($this,$mode) $fname
}

itcl::body myGUI::CreateGUI { base } {

    set basewidget $base
    $this Reset 0

    set labellist ""
    
    for { set i 1 } { $i <= 3 } { incr i } {
    
	set parent [ frame $basewidget.$i ]
	pack $parent -side top -expand false -fill x

	switch -exact $i {
	    "1" { 
		set name "Input File:"
		set mode readname
	    }
	    "2" {
		set name "Save File :"
		set mode writename
	    }
	    "3" {
		set name "Directory :"
		set mode directory
	    }
	}
	
	eval "iwidgets::entryfield $parent.1 -labeltext \"$name\"  -width 20  -textvariable [ itcl::scope parameters($this,$mode) ] -relief sunken"
	lappend labellist $parent.1
	eval "button $parent.3 -text \"...\" -command { $this GetName $mode }"
	pack $parent.3 -side right -expand false -padx 5
	pack $parent.1 -side left  -expand true -fill x -padx 2
    }

    eval "iwidgets::Labeledwidget::alignlabels $labellist" 

    set w [ frame $basewidget.bottom ]
    pack $w -side bottom -expand true -fill x -pady 10 -padx 20

    eval "button $w.1 -text Reset -command { $this Reset 1 }"
    eval "button $w.2 -text Color -command { $this SetColor $basewidget }"
    button $w.3 -text Exit -command { exit }
    pack  $w.1 $w.2 $w.3 -side left -expand true -fill x -padx 2
}

# ----------------------------------------------------------------------------------


frame .gui
pack .gui -side top -expand true -fill both

set element [ myGUI \#auto .gui ]
